import MetalKit
import SwiftUI

final class VexilUIView: UIView {
    private let _mtkView: MTKView!
    private let _renderer: VexilMain!

    init() {
        guard let defaultDevice = MTLCreateSystemDefaultDevice() else {
            fatalError("This doesn't have a suitable GPU!")
        }
        _mtkView = MTKView(frame: .zero, device: defaultDevice)
 
        _mtkView.backgroundColor = UIColor.clear

        guard let newRenderer = VexilMain(metalKitView: _mtkView) else {
            fatalError("Renderer cannot be initialized")
        }

        _renderer = newRenderer

        _renderer.mtkView(_mtkView, drawableSizeWillChange: _mtkView.drawableSize)

        super.init(frame: .zero)
        self.addSubview(_mtkView)
        _mtkView.translatesAutoresizingMaskIntoConstraints = false
        _mtkView.leadingAnchor.constraint(equalTo: self.leadingAnchor).isActive = true
        _mtkView.trailingAnchor.constraint(equalTo: self.trailingAnchor).isActive = true
        _mtkView.topAnchor.constraint(equalTo: self.topAnchor).isActive = true
        _mtkView.bottomAnchor.constraint(equalTo: self.bottomAnchor).isActive = true
        _mtkView.delegate = _renderer
    }

    required init?(coder: NSCoder) {
        fatalError("init(coder:) has not been implemented")
    }
    
    override func touchesBegan(_ touches: Set<UITouch>, with event: UIEvent?) {
        _renderer.touchesBegan(touches,with: event)
    }
    
    override func touchesMoved(_ touches: Set<UITouch>, with event: UIEvent?) {
        _renderer.touchesMoved(touches,with: event)
    }
    
    override func touchesEnded(_ touches: Set<UITouch>, with event: UIEvent?) {
        _renderer.touchesEnded(touches,with: event)
    }
}

struct VexilRepresentable: UIViewRepresentable {
    func makeUIView(context: UIViewRepresentableContext<VexilRepresentable>) -> VexilUIView {
        return VexilUIView()
    }

    func updateUIView(_ uiView: VexilUIView, context: UIViewRepresentableContext<VexilRepresentable>) {
    }
}
